-- cards	- uses sprite sheet for source of cards
-- functions available for calling:
--		deck,order	= cards:createDeck()			-- returns 52 cards and random order of 52 numbers
--		cards.deal( curX, curY, destX, destY, rate )-- simulated card dealing
--		cards.draw(card,x,y)						-- draws single card
--		hands = cards.create_hands(#hands,#cards)	-- returns hands (defaults to 4 hands, 13 cards each)
--		cards.fan_cards( hand,x,y )					-- display a hand as a fan

local gr = love.graphics

local cards = {}

function cards:createDeck()
	self.image = gr.newImage( 'playingcards.png' )
	-- location of cards on sprite sheet (more complex because of sprite sheet flaw)
	local club = {}		-- clubs
		for i = 1, 13 do club[i] = {(i-1)*73+39, 11,70,95} end
	local spad={}		-- spades
		for i = 1, 13 do spad[i] = {(i-1)*73+39,116,70,95} end
	local hear = {}		-- hearts
		for i = 1, 13 do hear[i] = {(i-1)*73+39,219,70,95} end	-- note: y offset not constant
	local diam = {}		-- diamonds
		for i = 1, 13 do diam[i] = {(i-1)*73+39,324,70,95} end
	local redback	= {118,435,68,95}
	local blueback	= {840,435,68,95}
	local joke		= {200,436,68,95}
	-- create quads for each card
	backred	 = self.create_quads( redback )
	backblue = self.create_quads( blueback )
	joker	 = self.create_quads( joke )
	clubquad = self.create_quads( club )
	spadquad = self.create_quads( spad )
	hearquad = self.create_quads( hear )
	diamquad = self.create_quads( diam )
	-- combine quads into a deck of cards (quads)
	local deck, id = {}, 0
	for i = 1, #club do id = id + 1 deck[id] = clubquad[i] end
	for i = 1, #diam do id = id + 1 deck[id] = diamquad[i] end
	for i = 1, #spad do id = id + 1 deck[id] = spadquad[i] end
	for i = 1, #hear do id = id + 1 deck[id] = hearquad[i] end
	-- shuffle deck (actually shuffle pointer into deck)
	local order = {}
	for i = 1, #deck do order[i] = i end	-- pointer to cards in deck (deck unchanged by sorting)
	self.shuffle( order )
	return deck, order
	end

function cards.create_quads(tab)
	local tmp = {}
	if type(tab[1])=='table' then
		for i=1,#tab do
			tmp[i] = gr.newQuad(tab[i][1], tab[i][2], tab[i][3], tab[i][4], cards.image:getWidth(), cards.image:getHeight())
			end
		else
		tmp = gr.newQuad(tab[1], tab[2], tab[3], tab[4], cards.image:getWidth(), cards.image:getHeight())
		end
	return tmp
	end

function cards.linear_deal( curT,starX,endX,starY,endY,endT )
	-- lapsed time, starting & ending values for x&y, time to end)
	local tx,bx,cx,dx = curT,starX,endX-starX,endT	-- convert to tweening notation
	local ty,by,cy,dy = curT,starY,endY-starY,endT
	if curT < endT then
		curX=math.floor(cx*tx/dx+bx)
		curY=math.floor(cy*ty/dy+by)
		end
	return curX, curY
	end

function cards.outExp_deal( curT,starX,endX,starY,endY,endT )
	-- lapsed time, starting & ending values for x&y, time to end)
	local tx,bx,cx,dx = curT,starX,endX-starX,endT	-- convert to tweening notation
	local ty,by,cy,dy = curT,starY,endY-starY,endT
	if curT < endT then
		curX=math.floor(cx * 1.001 * (-math.pow(2, -10 * tx / dx) + 1) + bx)
		curY=math.floor(cy * 1.001 * (-math.pow(2, -10 * ty / dy) + 1) + by)
		end
	return curX, curY
	end

function cards.outQuint_deal( curT,starX,endX,starY,endY,endT )
	-- lapsed time, starting & ending values for x&y, time to end)
	local tx,bx,cx,dx = curT,starX,endX-starX,endT	-- convert to tweening notation
	local ty,by,cy,dy = curT,starY,endY-starY,endT
	if curT < endT then
		curX=math.floor(cx * (math.pow(tx / dx - 1, 5) + 1) + bx )
		curY=math.floor(cy * (math.pow(ty / dy - 1, 5) + 1) + by )
		end
	return curX, curY
	end
function cards.outQuad_deal( curT,starX,endX,starY,endY,endT )
	-- lapsed time, starting & ending values for x&y, time to end)
	local tx,bx,cx,dx = curT/endT,starX,endX-starX,endT	-- convert to tweening notation
	local ty,by,cy,dy = curT/endT,starY,endY-starY,endT
	if curT < endT then
		curX=math.floor(-cx * tx * (tx - 2) + bx )
		curY=math.floor(-cy * ty * (ty - 2) + by )
		end
	return curX, curY
	end

function cards.create_hands( nh, nc )
	local hand = {}
	local numHands = nh or 4
	local numCards = nc or 13
	for j = 1,numHands do
		hand[j] = {}
		for i = 1,numCards do
			hand[j][i] = order[ numCards * (j-1) + i ]
			end
		table.sort( hand[j] )
		end
	return hand
	end

function cards.show_hands(h)
	for j = 1, #h do
		for i = 1, #h[j] do
			gr.drawq( cards.image, deck[h[j][i]], 35*(i-1)+5, 100*j, 0, 1, 1 )
			end
		end
	end

function cards.fan_cards( h, x, y )	-- hand of cards, position to display
	local a = {}
	for i = 1, #h do a[i] = math.pi*(i-3-#h/2)/20 end
	for i = 1, #h do
		gr.drawq( cards.image, deck[h[i]], x, y, a[i], 1, 1, 0, 95 )
		end
	end

function cards.shuffle( tab )
	local n = #tab
	while n >= 2 do
		local k = math.random( n )
		tab[n], tab[k] = tab[k], tab[n]
		n = n - 1
		end
	end

function cards.draw(card,x,y,r,sx,sy,ox,oy)
	r=r or 0
	sx, sy = sx or 1, sy or 1
	ox, oy = ox or 0, oy or 0
	gr.drawq(cards.image,card,x,y,r,sx,sy,ox,oy)
	end

return cards
